@@ -1,51 +1,48 @@
 #!/usr/bin/env python3
 
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import argparse
+import faulthandler
 import os
 import sys
 
 from UM.Platform import Platform
+from cura.ApplicationMetadata import CuraAppName
 
 parser = argparse.ArgumentParser(prog = "cura",
                                  add_help = False)
-parser.add_argument('--debug',
-                    action='store_true',
+parser.add_argument("--debug",
+                    action="store_true",
                     default = False,
                     help = "Turn on the debug mode by setting this option."
                     )
-parser.add_argument('--trigger-early-crash',
-                    dest = 'trigger_early_crash',
-                    action = 'store_true',
-                    default = False,
-                    help = "FOR TESTING ONLY. Trigger an early crash to show the crash dialog."
-                    )
 known_args = vars(parser.parse_known_args()[0])
 
 if not known_args["debug"]:
     def get_cura_dir_path():
         if Platform.isWindows():
-            return os.path.expanduser("~/AppData/Roaming/cura")
+            appdata_path = os.getenv("APPDATA")
+            if not appdata_path: #Defensive against the environment variable missing (should never happen).
+                appdata_path = "."
+            return os.path.join(appdata_path, CuraAppName)
         elif Platform.isLinux():
-            return os.path.expanduser("~/.local/share/cura")
+            return os.path.expanduser("~/.local/share/" + CuraAppName)
         elif Platform.isOSX():
-            return os.path.expanduser("~/Library/Logs/cura")
+            return os.path.expanduser("~/Library/Logs/" + CuraAppName)
 
-    if hasattr(sys, "frozen"):
+    # Do not redirect stdout and stderr to files if we are running CLI.
+    if hasattr(sys, "frozen") and "cli" not in os.path.basename(sys.argv[0]).lower():
         dirpath = get_cura_dir_path()
         os.makedirs(dirpath, exist_ok = True)
         sys.stdout = open(os.path.join(dirpath, "stdout.log"), "w", encoding = "utf-8")
         sys.stderr = open(os.path.join(dirpath, "stderr.log"), "w", encoding = "utf-8")
 
-import platform
-import faulthandler
 
-#WORKAROUND: GITHUB-88 GITHUB-385 GITHUB-612
+# WORKAROUND: GITHUB-88 GITHUB-385 GITHUB-612
 if Platform.isLinux(): # Needed for platform.linux_distribution, which is not available on Windows and OSX
     # For Ubuntu: https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
-    linux_distro_name = platform.linux_distribution()[0].lower()
     # The workaround is only needed on Ubuntu+NVidia drivers. Other drivers are not affected, but fine with this fix.
     try:
         import ctypes
@@ -63,6 +60,14 @@ if Platform.isWindows() and hasattr(sys, "frozen"):
     except KeyError:
         pass
 
+# GITHUB issue #6194: https://github.com/Ultimaker/Cura/issues/6194
+# With AppImage 2 on Linux, the current working directory will be somewhere in /tmp/<rand>/usr, which is owned
+# by root. For some reason, QDesktopServices.openUrl() requires to have a usable current working directory,
+# otherwise it doesn't work. This is a workaround on Linux that before we call QDesktopServices.openUrl(), we
+# switch to a directory where the user has the ownership.
+if Platform.isLinux() and hasattr(sys, "frozen"):
+    os.chdir(os.path.expanduser("~"))
+
 # WORKAROUND: GITHUB-704 GITHUB-708
 # It looks like setuptools creates a .pth file in
 # the default /usr/lib which causes the default site-packages
@@ -79,6 +84,7 @@ if "PYTHONPATH" in os.environ.keys():                       # If PYTHONPATH is u
             sys.path.remove(PATH_real)
         sys.path.insert(1, PATH_real)                       # Insert it at 1 after os.curdir, which is 0.
 
+
 def exceptHook(hook_type, value, traceback):
     from cura.CrashHandler import CrashHandler
     from cura.CuraApplication import CuraApplication
@@ -121,25 +127,54 @@ def exceptHook(hook_type, value, traceback):
         _crash_handler.early_crash_dialog.show()
         sys.exit(application.exec_())
 
-if not known_args["debug"]:
-    sys.excepthook = exceptHook
+
+# Set exception hook to use the crash dialog handler
+sys.excepthook = exceptHook
+# Enable dumping traceback for all threads
+if sys.stderr:
+    faulthandler.enable(file = sys.stderr, all_threads = True)
+else:
+    faulthandler.enable(file = sys.stdout, all_threads = True)
 
 # Workaround for a race condition on certain systems where there
 # is a race condition between Arcus and PyQt. Importing Arcus
 # first seems to prevent Sip from going into a state where it
 # tries to create PyQt objects on a non-main thread.
 import Arcus #@UnusedImport
-import cura.CuraApplication
-import cura.Settings.CuraContainerRegistry
+import Savitar #@UnusedImport
+from cura.CuraApplication import CuraApplication
 
-faulthandler.enable()
 
-# Force an instance of CuraContainerRegistry to be created and reused later.
-cura.Settings.CuraContainerRegistry.CuraContainerRegistry.getInstance()
+# WORKAROUND: CURA-6739
+# The CTM file loading module in Trimesh requires the OpenCTM library to be dynamically loaded. It uses
+# ctypes.util.find_library() to find libopenctm.dylib, but this doesn't seem to look in the ".app" application folder
+# on Mac OS X. Adding the search path to environment variables such as DYLD_LIBRARY_PATH and DYLD_FALLBACK_LIBRARY_PATH
+# makes it work. The workaround here uses DYLD_FALLBACK_LIBRARY_PATH.
+if Platform.isOSX() and getattr(sys, "frozen", False):
+    old_env = os.environ.get("DYLD_FALLBACK_LIBRARY_PATH", "")
+    # This is where libopenctm.so is in the .app folder.
+    search_path = os.path.join(CuraApplication.getInstallPrefix(), "MacOS")
+    path_list = old_env.split(":")
+    if search_path not in path_list:
+        path_list.append(search_path)
+    os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = ":".join(path_list)
+    import trimesh.exchange.load
+    os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = old_env
 
-# This pre-start up check is needed to determine if we should start the application at all.
-if not cura.CuraApplication.CuraApplication.preStartUp(parser = parser, parsed_command_line = known_args):
-    sys.exit(0)
+# WORKAROUND: CURA-6739
+# Similar CTM file loading fix for Linux, but NOTE THAT this doesn't work directly with Python 3.5.7. There's a fix
+# for ctypes.util.find_library() in Python 3.6 and 3.7. That fix makes sure that find_library() will check
+# LD_LIBRARY_PATH. With Python 3.5, that fix needs to be backported to make this workaround work.
+if Platform.isLinux() and getattr(sys, "frozen", False):
+    old_env = os.environ.get("LD_LIBRARY_PATH", "")
+    # This is where libopenctm.so is in the AppImage.
+    search_path = os.path.join(CuraApplication.getInstallPrefix(), "bin")
+    path_list = old_env.split(":")
+    if search_path not in path_list:
+        path_list.append(search_path)
+    os.environ["LD_LIBRARY_PATH"] = ":".join(path_list)
+    import trimesh.exchange.load
+    os.environ["LD_LIBRARY_PATH"] = old_env
 
-app = cura.CuraApplication.CuraApplication.getInstance(parser = parser, parsed_command_line = known_args)
+app = CuraApplication()
 app.run()
