@@ -1,21 +1,26 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import numpy
 from string import Formatter
 from enum import IntEnum
 import time
+from typing import Any, cast, Dict, List, Optional, Set
 import re
+import Arcus #For typing.
 
 from UM.Job import Job
-from UM.Application import Application
 from UM.Logger import Logger
+from UM.Scene.SceneNode import SceneNode
+from UM.Settings.ContainerStack import ContainerStack #For typing.
+from UM.Settings.SettingRelation import SettingRelation #For typing.
 
 from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
-
+from UM.Scene.Scene import Scene #For typing.
 from UM.Settings.Validator import ValidatorState
 from UM.Settings.SettingRelation import RelationType
 
+from cura.CuraApplication import CuraApplication
 from cura.Scene.CuraSceneNode import CuraSceneNode
 from cura.OneAtATimeIterator import OneAtATimeIterator
 from cura.Settings.ExtruderManager import ExtruderManager
@@ -32,100 +37,119 @@ class StartJobResult(IntEnum):
     MaterialIncompatible = 5
     BuildPlateError = 6
     ObjectSettingError = 7 #When an error occurs in per-object settings.
+    ObjectsWithDisabledExtruder = 8
 
 
-##  Formatter class that handles token expansion in start/end gcod
+##  Formatter class that handles token expansion in start/end gcode
 class GcodeStartEndFormatter(Formatter):
-    def get_value(self, key, args, kwargs):  # [CodeStyle: get_value is an overridden function from the Formatter class]
+    def __init__(self, default_extruder_nr: int = -1) -> None:
+        super().__init__()
+        self._default_extruder_nr = default_extruder_nr
+
+    def get_value(self, key: str, args: str, kwargs: dict) -> str: #type: ignore # [CodeStyle: get_value is an overridden function from the Formatter class]
         # The kwargs dictionary contains a dictionary for each stack (with a string of the extruder_nr as their key),
         # and a default_extruder_nr to use when no extruder_nr is specified
 
-        if isinstance(key, str):
+        extruder_nr = self._default_extruder_nr
+
+        key_fragments = [fragment.strip() for fragment in key.split(",")]
+        if len(key_fragments) == 2:
             try:
-                extruder_nr = kwargs["default_extruder_nr"]
+                extruder_nr = int(key_fragments[1])
             except ValueError:
-                extruder_nr = -1
-
-            key_fragments = [fragment.strip() for fragment in key.split(',')]
-            if len(key_fragments) == 2:
                 try:
-                    extruder_nr = int(key_fragments[1])
-                except ValueError:
-                    try:
-                        extruder_nr = int(kwargs["-1"][key_fragments[1]]) # get extruder_nr values from the global stack
-                    except (KeyError, ValueError):
-                        # either the key does not exist, or the value is not an int
-                        Logger.log("w", "Unable to determine stack nr '%s' for key '%s' in start/end g-code, using global stack", key_fragments[1], key_fragments[0])
-            elif len(key_fragments) != 1:
-                Logger.log("w", "Incorrectly formatted placeholder '%s' in start/end g-code", key)
-                return "{" + str(key) + "}"
-
-            key = key_fragments[0]
-            try:
-                return kwargs[str(extruder_nr)][key]
-            except KeyError:
-                Logger.log("w", "Unable to replace '%s' placeholder in start/end g-code", key)
-                return "{" + key + "}"
-        else:
+                    extruder_nr = int(kwargs["-1"][key_fragments[1]]) # get extruder_nr values from the global stack #TODO: How can you ever provide the '-1' kwarg?
+                except (KeyError, ValueError):
+                    # either the key does not exist, or the value is not an int
+                    Logger.log("w", "Unable to determine stack nr '%s' for key '%s' in start/end g-code, using global stack", key_fragments[1], key_fragments[0])
+        elif len(key_fragments) != 1:
             Logger.log("w", "Incorrectly formatted placeholder '%s' in start/end g-code", key)
-            return "{" + str(key) + "}"
+            return "{" + key + "}"
+
+        key = key_fragments[0]
+
+        default_value_str = "{" + key + "}"
+        value = default_value_str
+        # "-1" is global stack, and if the setting value exists in the global stack, use it as the fallback value.
+        if key in kwargs["-1"]:
+            value = kwargs["-1"][key]
+        if str(extruder_nr) in kwargs and key in kwargs[str(extruder_nr)]:
+            value = kwargs[str(extruder_nr)][key]
+
+        if value == default_value_str:
+            Logger.log("w", "Unable to replace '%s' placeholder in start/end g-code", key)
+
+        return value
 
 
 ##  Job class that builds up the message of scene data to send to CuraEngine.
 class StartSliceJob(Job):
-    def __init__(self, slice_message):
+    def __init__(self, slice_message: Arcus.PythonMessage) -> None:
         super().__init__()
 
-        self._scene = Application.getInstance().getController().getScene()
-        self._slice_message = slice_message
-        self._is_cancelled = False
-        self._build_plate_number = None
+        self._scene = CuraApplication.getInstance().getController().getScene() #type: Scene
+        self._slice_message = slice_message #type: Arcus.PythonMessage
+        self._is_cancelled = False #type: bool
+        self._build_plate_number = None #type: Optional[int]
 
-        self._all_extruders_settings = None # cache for all setting values from all stacks (global & extruder) for the current machine
+        self._all_extruders_settings = None #type: Optional[Dict[str, Any]] # cache for all setting values from all stacks (global & extruder) for the current machine
 
-    def getSliceMessage(self):
+    def getSliceMessage(self) -> Arcus.PythonMessage:
         return self._slice_message
 
-    def setBuildPlate(self, build_plate_number):
+    def setBuildPlate(self, build_plate_number: int) -> None:
         self._build_plate_number = build_plate_number
 
     ##  Check if a stack has any errors.
     ##  returns true if it has errors, false otherwise.
-    def _checkStackForErrors(self, stack):
+    def _checkStackForErrors(self, stack: ContainerStack) -> bool:
         if stack is None:
             return False
 
+        # if there are no per-object settings we don't need to check the other settings here
+        stack_top = stack.getTop()
+        if stack_top is None or not stack_top.getAllKeys():
+            return False
+
         for key in stack.getAllKeys():
             validation_state = stack.getProperty(key, "validationState")
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError):
+            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
                 Logger.log("w", "Setting %s is not valid, but %s. Aborting slicing.", key, validation_state)
                 return True
             Job.yieldThread()
         return False
 
     ##  Runs the job that initiates the slicing.
-    def run(self):
+    def run(self) -> None:
         if self._build_plate_number is None:
             self.setResult(StartJobResult.Error)
             return
 
-        stack = Application.getInstance().getGlobalContainerStack()
+        stack = CuraApplication.getInstance().getGlobalContainerStack()
         if not stack:
             self.setResult(StartJobResult.Error)
             return
 
         # Don't slice if there is a setting with an error value.
-        if Application.getInstance().getMachineManager().stacksHaveErrors:
+        if CuraApplication.getInstance().getMachineManager().stacksHaveErrors:
             self.setResult(StartJobResult.SettingError)
             return
 
-        if Application.getInstance().getBuildVolume().hasErrors():
+        if CuraApplication.getInstance().getBuildVolume().hasErrors():
             self.setResult(StartJobResult.BuildPlateError)
             return
 
+        # Wait for error checker to be done.
+        while CuraApplication.getInstance().getMachineErrorChecker().needToWaitForResult:
+            time.sleep(0.1)
+
+        if CuraApplication.getInstance().getMachineErrorChecker().hasError:
+            self.setResult(StartJobResult.SettingError)
+            return
+
         # Don't slice if the buildplate or the nozzle type is incompatible with the materials
-        if not Application.getInstance().getMachineManager().variantBuildplateCompatible and \
-                not Application.getInstance().getMachineManager().variantBuildplateUsable:
+        if not CuraApplication.getInstance().getMachineManager().variantBuildplateCompatible and \
+                not CuraApplication.getInstance().getMachineManager().variantBuildplateUsable:
             self.setResult(StartJobResult.MaterialIncompatible)
             return
 
@@ -152,7 +176,8 @@ class StartSliceJob(Job):
             # Remove old layer data.
             for node in DepthFirstIterator(self._scene.getRoot()):
                 if node.callDecoration("getLayerData") and node.callDecoration("getBuildPlateNumber") == self._build_plate_number:
-                    node.getParent().removeChild(node)
+                    # Singe we walk through all nodes in the scene, they always have a parent.
+                    cast(SceneNode, node.getParent()).removeChild(node)
                     break
 
             # Get the objects in their groups to print.
@@ -173,7 +198,8 @@ class StartSliceJob(Job):
                     children = node.getAllChildren()
                     children.append(node)
                     for child_node in children:
-                        if child_node.getMeshData() and child_node.getMeshData().getVertices() is not None:
+                        mesh_data = child_node.getMeshData()
+                        if mesh_data and mesh_data.getVertices() is not None:
                             temp_list.append(child_node)
 
                     if temp_list:
@@ -185,11 +211,9 @@ class StartSliceJob(Job):
                 temp_list = []
                 has_printing_mesh = False
                 for node in DepthFirstIterator(self._scene.getRoot()):
-                    if node.callDecoration("isSliceable") and node.getMeshData() and node.getMeshData().getVertices() is not None:
-                        per_object_stack = node.callDecoration("getStack")
-                        is_non_printing_mesh = False
-                        if per_object_stack:
-                            is_non_printing_mesh = any(per_object_stack.getProperty(key, "value") for key in NON_PRINTING_MESH_SETTINGS)
+                    mesh_data = node.getMeshData()
+                    if node.callDecoration("isSliceable") and mesh_data and mesh_data.getVertices() is not None:
+                        is_non_printing_mesh = bool(node.callDecoration("isNonPrintingMesh"))
 
                         # Find a reason not to add the node
                         if node.callDecoration("getBuildPlateNumber") != self._build_plate_number:
@@ -203,7 +227,7 @@ class StartSliceJob(Job):
 
                     Job.yieldThread()
 
-                #If the list doesn't have any model with suitable settings then clean the list
+                # If the list doesn't have any model with suitable settings then clean the list
                 # otherwise CuraEngine will crash
                 if not has_printing_mesh:
                     temp_list.clear()
@@ -211,18 +235,33 @@ class StartSliceJob(Job):
                 if temp_list:
                     object_groups.append(temp_list)
 
-            extruders_enabled = {position: stack.isEnabled for position, stack in Application.getInstance().getGlobalContainerStack().extruders.items()}
+            global_stack = CuraApplication.getInstance().getGlobalContainerStack()
+            if not global_stack:
+                return
+            extruders_enabled = {position: stack.isEnabled for position, stack in global_stack.extruders.items()}
             filtered_object_groups = []
+            has_model_with_disabled_extruders = False
+            associated_disabled_extruders = set()
             for group in object_groups:
-                stack = Application.getInstance().getGlobalContainerStack()
+                stack = global_stack
                 skip_group = False
                 for node in group:
-                    if not extruders_enabled[node.callDecoration("getActiveExtruderPosition")]:
+                    # Only check if the printing extruder is enabled for printing meshes
+                    is_non_printing_mesh = node.callDecoration("evaluateIsNonPrintingMesh")
+                    extruder_position = node.callDecoration("getActiveExtruderPosition")
+                    if not is_non_printing_mesh and not extruders_enabled[extruder_position]:
                         skip_group = True
-                        break
+                        has_model_with_disabled_extruders = True
+                        associated_disabled_extruders.add(extruder_position)
                 if not skip_group:
                     filtered_object_groups.append(group)
 
+            if has_model_with_disabled_extruders:
+                self.setResult(StartJobResult.ObjectsWithDisabledExtruder)
+                associated_disabled_extruders = {str(c) for c in sorted([int(p) + 1 for p in associated_disabled_extruders])}
+                self.setMessage(", ".join(associated_disabled_extruders))
+                return
+
             # There are cases when there is nothing to slice. This can happen due to one at a time slicing not being
             # able to find a possible sequence or because there are no objects on the build plate (or they are outside
             # the build volume)
@@ -234,15 +273,19 @@ class StartSliceJob(Job):
             self._buildGlobalInheritsStackMessage(stack)
 
             # Build messages for extruder stacks
-            for extruder_stack in ExtruderManager.getInstance().getMachineExtruders(stack.getId()):
+            for extruder_stack in global_stack.extruderList:
                 self._buildExtruderMessage(extruder_stack)
 
             for group in filtered_object_groups:
                 group_message = self._slice_message.addRepeatedMessage("object_lists")
-                if group[0].getParent() is not None and group[0].getParent().callDecoration("isGroup"):
-                    self._handlePerObjectSettings(group[0].getParent(), group_message)
+                parent = group[0].getParent()
+                if parent is not None and parent.callDecoration("isGroup"):
+                    self._handlePerObjectSettings(cast(CuraSceneNode, parent), group_message)
+
                 for object in group:
                     mesh_data = object.getMeshData()
+                    if mesh_data is None:
+                        continue
                     rot_scale = object.getWorldTransformation().getTransposed().getData()[0:3, 0:3]
                     translate = object.getWorldTransformation().getData()[:3, 3]
 
@@ -257,7 +300,7 @@ class StartSliceJob(Job):
 
                     obj = group_message.addRepeatedMessage("objects")
                     obj.id = id(object)
-
+                    obj.name = object.getName()
                     indices = mesh_data.getIndices()
                     if indices is not None:
                         flat_verts = numpy.take(verts, indices.flatten(), axis=0)
@@ -266,19 +309,22 @@ class StartSliceJob(Job):
 
                     obj.vertices = flat_verts
 
-                    self._handlePerObjectSettings(object, obj)
+                    self._handlePerObjectSettings(cast(CuraSceneNode, object), obj)
 
                     Job.yieldThread()
 
         self.setResult(StartJobResult.Finished)
 
-    def cancel(self):
+    def cancel(self) -> None:
         super().cancel()
         self._is_cancelled = True
 
-    def isCancelled(self):
+    def isCancelled(self) -> bool:
         return self._is_cancelled
 
+    def setIsCancelled(self, value: bool):
+        self._is_cancelled = value
+
     ##  Creates a dictionary of tokens to replace in g-code pieces.
     #
     #   This indicates what should be replaced in the start and end g-codes.
@@ -286,49 +332,49 @@ class StartSliceJob(Job):
     #   with.
     #   \return A dictionary of replacement tokens to the values they should be
     #   replaced with.
-    def _buildReplacementTokens(self, stack) -> dict:
-        default_extruder_position = int(Application.getInstance().getMachineManager().defaultExtruderPosition)
+    def _buildReplacementTokens(self, stack: ContainerStack) -> Dict[str, Any]:
         result = {}
         for key in stack.getAllKeys():
-            setting_type = stack.definition.getProperty(key, "type")
             value = stack.getProperty(key, "value")
-            if setting_type == "extruder" and value == -1:
-                # replace with the default value
-                value = default_extruder_position
             result[key] = value
             Job.yieldThread()
 
         result["print_bed_temperature"] = result["material_bed_temperature"] # Renamed settings.
         result["print_temperature"] = result["material_print_temperature"]
+        result["travel_speed"] = result["speed_travel"]
         result["time"] = time.strftime("%H:%M:%S") #Some extra settings.
         result["date"] = time.strftime("%d-%m-%Y")
         result["day"] = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"][int(time.strftime("%w"))]
 
-        initial_extruder_stack = Application.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
+        initial_extruder_stack = CuraApplication.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
         initial_extruder_nr = initial_extruder_stack.getProperty("extruder_nr", "value")
         result["initial_extruder_nr"] = initial_extruder_nr
 
         return result
 
+    def _cacheAllExtruderSettings(self):
+        global_stack = cast(ContainerStack, CuraApplication.getInstance().getGlobalContainerStack())
+
+        # NB: keys must be strings for the string formatter
+        self._all_extruders_settings = {
+            "-1": self._buildReplacementTokens(global_stack)
+        }
+        for extruder_stack in ExtruderManager.getInstance().getActiveExtruderStacks():
+            extruder_nr = extruder_stack.getProperty("extruder_nr", "value")
+            self._all_extruders_settings[str(extruder_nr)] = self._buildReplacementTokens(extruder_stack)
+
     ##  Replace setting tokens in a piece of g-code.
     #   \param value A piece of g-code to replace tokens in.
     #   \param default_extruder_nr Stack nr to use when no stack nr is specified, defaults to the global stack
-    def _expandGcodeTokens(self, value: str, default_extruder_nr: int = -1):
+    def _expandGcodeTokens(self, value: str, default_extruder_nr: int = -1) -> str:
         if not self._all_extruders_settings:
-            global_stack = Application.getInstance().getGlobalContainerStack()
-
-            # NB: keys must be strings for the string formatter
-            self._all_extruders_settings = {
-                "-1": self._buildReplacementTokens(global_stack)
-            }
-
-            for extruder_stack in ExtruderManager.getInstance().getMachineExtruders(global_stack.getId()):
-                extruder_nr = extruder_stack.getProperty("extruder_nr", "value")
-                self._all_extruders_settings[str(extruder_nr)] = self._buildReplacementTokens(extruder_stack)
+            self._cacheAllExtruderSettings()
 
         try:
             # any setting can be used as a token
-            fmt = GcodeStartEndFormatter()
+            fmt = GcodeStartEndFormatter(default_extruder_nr = default_extruder_nr)
+            if self._all_extruders_settings is None:
+                return ""
             settings = self._all_extruders_settings.copy()
             settings["default_extruder_nr"] = default_extruder_nr
             return str(fmt.format(value, **settings))
@@ -337,11 +383,17 @@ class StartSliceJob(Job):
             return str(value)
 
     ##  Create extruder message from stack
-    def _buildExtruderMessage(self, stack):
+    def _buildExtruderMessage(self, stack: ContainerStack) -> None:
         message = self._slice_message.addRepeatedMessage("extruders")
         message.id = int(stack.getMetaDataEntry("position"))
+        if not self._all_extruders_settings:
+            self._cacheAllExtruderSettings()
 
-        settings = self._buildReplacementTokens(stack)
+        if self._all_extruders_settings is None:
+            return
+
+        extruder_nr = stack.getProperty("extruder_nr", "value")
+        settings = self._all_extruders_settings[str(extruder_nr)].copy()
 
         # Also send the material GUID. This is a setting in fdmprinter, but we have no interface for it.
         settings["material_guid"] = stack.material.getMetaDataEntry("GUID", "")
@@ -364,8 +416,14 @@ class StartSliceJob(Job):
     #
     #   The settings are taken from the global stack. This does not include any
     #   per-extruder settings or per-object settings.
-    def _buildGlobalSettingsMessage(self, stack):
-        settings = self._buildReplacementTokens(stack)
+    def _buildGlobalSettingsMessage(self, stack: ContainerStack) -> None:
+        if not self._all_extruders_settings:
+            self._cacheAllExtruderSettings()
+
+        if self._all_extruders_settings is None:
+            return
+
+        settings = self._all_extruders_settings["-1"].copy()
 
         # Pre-compute material material_bed_temp_prepend and material_print_temp_prepend
         start_gcode = settings["machine_start_gcode"]
@@ -378,7 +436,7 @@ class StartSliceJob(Job):
 
         # Replace the setting tokens in start and end g-code.
         # Use values from the first used extruder by default so we get the expected temperatures
-        initial_extruder_stack = Application.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
+        initial_extruder_stack = CuraApplication.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
         initial_extruder_nr = initial_extruder_stack.getProperty("extruder_nr", "value")
 
         settings["machine_start_gcode"] = self._expandGcodeTokens(settings["machine_start_gcode"], initial_extruder_nr)
@@ -399,7 +457,7 @@ class StartSliceJob(Job):
     #
     #   \param stack The global stack with all settings, from which to read the
     #   limit_to_extruder property.
-    def _buildGlobalInheritsStackMessage(self, stack):
+    def _buildGlobalInheritsStackMessage(self, stack: ContainerStack) -> None:
         for key in stack.getAllKeys():
             extruder_position = int(round(float(stack.getProperty(key, "limit_to_extruder"))))
             if extruder_position >= 0:  # Set to a specific extruder.
@@ -409,9 +467,9 @@ class StartSliceJob(Job):
             Job.yieldThread()
 
     ##  Check if a node has per object settings and ensure that they are set correctly in the message
-    #   \param node \type{SceneNode} Node to check.
+    #   \param node Node to check.
     #   \param message object_lists message to put the per object settings in
-    def _handlePerObjectSettings(self, node, message):
+    def _handlePerObjectSettings(self, node: CuraSceneNode, message: Arcus.PythonMessage):
         stack = node.callDecoration("getStack")
 
         # Check if the node has a stack attached to it and the stack has any settings in the top container.
@@ -420,7 +478,7 @@ class StartSliceJob(Job):
 
         # Check all settings for relations, so we can also calculate the correct values for dependent settings.
         top_of_stack = stack.getTop()  # Cache for efficiency.
-        changed_setting_keys = set(top_of_stack.getAllKeys())
+        changed_setting_keys = top_of_stack.getAllKeys()
 
         # Add all relations to changed settings as well.
         for key in top_of_stack.getAllKeys():
@@ -429,8 +487,7 @@ class StartSliceJob(Job):
             Job.yieldThread()
 
         # Ensure that the engine is aware what the build extruder is.
-        if stack.getProperty("machine_extruder_count", "value") > 1:
-            changed_setting_keys.add("extruder_nr")
+        changed_setting_keys.add("extruder_nr")
 
         # Get values for all changed settings
         for key in changed_setting_keys:
@@ -449,9 +506,9 @@ class StartSliceJob(Job):
             Job.yieldThread()
 
     ##  Recursive function to put all settings that require each other for value changes in a list
-    #   \param relations_set \type{set} Set of keys (strings) of settings that are influenced
+    #   \param relations_set Set of keys of settings that are influenced
     #   \param relations list of relation objects that need to be checked.
-    def _addRelations(self, relations_set, relations):
+    def _addRelations(self, relations_set: Set[str], relations: List[SettingRelation]):
         for relation in filter(lambda r: r.role == "value" or r.role == "limit_to_extruder", relations):
             if relation.type == RelationType.RequiresTarget:
                 continue
