@@ -1,68 +1,74 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
-from UM.Application import Application
+import math
+import re
+from typing import Dict, List, NamedTuple, Optional, Union, Set
+
+import numpy
+
 from UM.Backend import Backend
 from UM.Job import Job
 from UM.Logger import Logger
-from UM.Math.AxisAlignedBox import AxisAlignedBox
 from UM.Math.Vector import Vector
 from UM.Message import Message
-from cura.Scene.CuraSceneNode import CuraSceneNode
 from UM.i18n import i18nCatalog
-from UM.Preferences import Preferences
 
-catalog = i18nCatalog("cura")
-
-from cura import LayerDataBuilder
+from cura.CuraApplication import CuraApplication
+from cura.LayerDataBuilder import LayerDataBuilder
 from cura.LayerDataDecorator import LayerDataDecorator
 from cura.LayerPolygon import LayerPolygon
+from cura.Scene.CuraSceneNode import CuraSceneNode
 from cura.Scene.GCodeListDecorator import GCodeListDecorator
 from cura.Settings.ExtruderManager import ExtruderManager
 
-import numpy
-import math
-import re
-from collections import namedtuple
+catalog = i18nCatalog("cura")
 
-# This parser is intented for interpret the common firmware codes among all the different flavors
+PositionOptional = NamedTuple("Position", [("x", Optional[float]), ("y", Optional[float]), ("z", Optional[float]), ("f", Optional[float]), ("e", Optional[float])])
+Position = NamedTuple("Position", [("x", float), ("y", float), ("z", float), ("f", float), ("e", List[float])])
+
+
+##  This parser is intended to interpret the common firmware codes among all the
+#   different flavors
 class FlavorParser:
 
-    def __init__(self):
-        Application.getInstance().hideMessageSignal.connect(self._onHideMessage)
+    def __init__(self) -> None:
+        CuraApplication.getInstance().hideMessageSignal.connect(self._onHideMessage)
         self._cancelled = False
-        self._message = None
+        self._message = None  # type: Optional[Message]
         self._layer_number = 0
         self._extruder_number = 0
+        # All extruder numbers that have been seen
+        self._extruders_seen = {0}  # type: Set[int]
         self._clearValues()
         self._scene_node = None
         # X, Y, Z position, F feedrate and E extruder values are stored
-        self._position = namedtuple('Position', ['x', 'y', 'z', 'f', 'e'])
+        self._position = Position
         self._is_layers_in_file = False  # Does the Gcode have the layers comment?
-        self._extruder_offsets = {}  # Offsets for multi extruders. key is index, value is [x-offset, y-offset]
+        self._extruder_offsets = {}  # type: Dict[int, List[float]] # Offsets for multi extruders. key is index, value is [x-offset, y-offset]
         self._current_layer_thickness = 0.2  # default
         self._filament_diameter = 2.85       # default
+        self._previous_extrusion_value = 0.0  # keep track of the filament retractions
 
-        Preferences.getInstance().addPreference("gcodereader/show_caution", True)
+        CuraApplication.getInstance().getPreferences().addPreference("gcodereader/show_caution", True)
 
-    def _clearValues(self):
+    def _clearValues(self) -> None:
         self._extruder_number = 0
-        self._extrusion_length_offset = [0]
+        self._extrusion_length_offset = [0] # type: List[float]
         self._layer_type = LayerPolygon.Inset0Type
         self._layer_number = 0
-        self._previous_z = 0
-        self._layer_data_builder = LayerDataBuilder.LayerDataBuilder()
-        self._center_is_zero = False
+        self._previous_z = 0 # type: float
+        self._layer_data_builder = LayerDataBuilder()
         self._is_absolute_positioning = True    # It can be absolute (G90) or relative (G91)
         self._is_absolute_extrusion = True  # It can become absolute (M82, default) or relative (M83)
 
     @staticmethod
-    def _getValue(line, code):
+    def _getValue(line: str, code: str) -> Optional[Union[str, int, float]]:
         n = line.find(code)
         if n < 0:
             return None
         n += len(code)
-        pattern = re.compile("[;\s]")
+        pattern = re.compile("[;\\s]")
         match = pattern.search(line, n)
         m = match.start() if match is not None else -1
         try:
@@ -72,29 +78,25 @@ class FlavorParser:
         except:
             return None
 
-    def _getInt(self, line, code):
+    def _getInt(self, line: str, code: str) -> Optional[int]:
         value = self._getValue(line, code)
         try:
-            return int(value)
+            return int(value)   # type: ignore
         except:
             return None
 
-    def _getFloat(self, line, code):
+    def _getFloat(self, line: str, code: str) -> Optional[float]:
         value = self._getValue(line, code)
         try:
-            return float(value)
+            return float(value) # type: ignore
         except:
             return None
 
-    def _onHideMessage(self, message):
+    def _onHideMessage(self, message: str) -> None:
         if message == self._message:
             self._cancelled = True
 
-    @staticmethod
-    def _getNullBoundingBox():
-        return AxisAlignedBox(minimum=Vector(0, 0, 0), maximum=Vector(10, 10, 10))
-
-    def _createPolygon(self, layer_thickness, path, extruder_offsets):
+    def _createPolygon(self, layer_thickness: float, path: List[List[Union[float, int]]], extruder_offsets: List[float]) -> bool:
         countvalid = 0
         for point in path:
             if point[5] > 0:
@@ -109,6 +111,8 @@ class FlavorParser:
             self._layer_data_builder.setLayerHeight(self._layer_number, path[0][2])
             self._layer_data_builder.setLayerThickness(self._layer_number, layer_thickness)
             this_layer = self._layer_data_builder.getLayer(self._layer_number)
+            if not this_layer:
+                return False
         except ValueError:
             return False
         count = len(path)
@@ -140,12 +144,12 @@ class FlavorParser:
         this_layer.polygons.append(this_poly)
         return True
 
-    def _createEmptyLayer(self, layer_number):
+    def _createEmptyLayer(self, layer_number: int) -> None:
         self._layer_data_builder.addLayer(layer_number)
         self._layer_data_builder.setLayerHeight(layer_number, 0)
         self._layer_data_builder.setLayerThickness(layer_number, 0)
 
-    def _calculateLineWidth(self, current_point, previous_point, current_extrusion, previous_extrusion, layer_thickness):
+    def _calculateLineWidth(self, current_point: Position, previous_point: Position, current_extrusion: float, previous_extrusion: float, layer_thickness: float) -> float:
         # Area of the filament
         Af = (self._filament_diameter / 2) ** 2 * numpy.pi
         # Length of the extruded filament
@@ -167,7 +171,7 @@ class FlavorParser:
             return 0.35
         return line_width
 
-    def _gCode0(self, position, params, path):
+    def _gCode0(self, position: Position, params: PositionOptional, path: List[List[Union[float, int]]]) -> Position:
         x, y, z, f, e = position
 
         if self._is_absolute_positioning:
@@ -185,6 +189,7 @@ class FlavorParser:
             new_extrusion_value = params.e if self._is_absolute_extrusion else e[self._extruder_number] + params.e
             if new_extrusion_value > e[self._extruder_number]:
                 path.append([x, y, z, f, new_extrusion_value + self._extrusion_length_offset[self._extruder_number], self._layer_type])  # extrusion
+                self._previous_extrusion_value = new_extrusion_value
             else:
                 path.append([x, y, z, f, new_extrusion_value + self._extrusion_length_offset[self._extruder_number], LayerPolygon.MoveRetractionType])  # retraction
             e[self._extruder_number] = new_extrusion_value
@@ -194,6 +199,8 @@ class FlavorParser:
             if z > self._previous_z and (z - self._previous_z < 1.5):
                 self._current_layer_thickness = z - self._previous_z # allow a tiny overlap
                 self._previous_z = z
+        elif self._previous_extrusion_value > e[self._extruder_number]:
+            path.append([x, y, z, f, e[self._extruder_number] + self._extrusion_length_offset[self._extruder_number], LayerPolygon.MoveRetractionType])
         else:
             path.append([x, y, z, f, e[self._extruder_number] + self._extrusion_length_offset[self._extruder_number], LayerPolygon.MoveCombingType])
         return self._position(x, y, z, f, e)
@@ -203,7 +210,7 @@ class FlavorParser:
     _gCode1 = _gCode0
 
     ##  Home the head.
-    def _gCode28(self, position, params, path):
+    def _gCode28(self, position: Position, params: PositionOptional, path: List[List[Union[float, int]]]) -> Position:
         return self._position(
             params.x if params.x is not None else position.x,
             params.y if params.y is not None else position.y,
@@ -212,24 +219,27 @@ class FlavorParser:
             position.e)
 
     ##  Set the absolute positioning
-    def _gCode90(self, position, params, path):
+    def _gCode90(self, position: Position, params: PositionOptional, path: List[List[Union[float, int]]]) -> Position:
         self._is_absolute_positioning = True
         self._is_absolute_extrusion = True
         return position
 
     ##  Set the relative positioning
-    def _gCode91(self, position, params, path):
+    def _gCode91(self, position: Position, params: PositionOptional, path: List[List[Union[float, int]]]) -> Position:
         self._is_absolute_positioning = False
         self._is_absolute_extrusion = False
         return position
 
     ##  Reset the current position to the values specified.
     #   For example: G92 X10 will set the X to 10 without any physical motion.
-    def _gCode92(self, position, params, path):
+    def _gCode92(self, position: Position, params: PositionOptional, path: List[List[Union[float, int]]]) -> Position:
         if params.e is not None:
             # Sometimes a G92 E0 is introduced in the middle of the GCode so we need to keep those offsets for calculate the line_width
             self._extrusion_length_offset[self._extruder_number] += position.e[self._extruder_number] - params.e
             position.e[self._extruder_number] = params.e
+            self._previous_extrusion_value = params.e
+        else:
+            self._previous_extrusion_value = 0.0
         return self._position(
             params.x if params.x is not None else position.x,
             params.y if params.y is not None else position.y,
@@ -237,7 +247,7 @@ class FlavorParser:
             params.f if params.f is not None else position.f,
             position.e)
 
-    def processGCode(self, G, line, position, path):
+    def processGCode(self, G: int, line: str, position: Position, path: List[List[Union[float, int]]]) -> Position:
         func = getattr(self, "_gCode%s" % G, None)
         line = line.split(";", 1)[0]  # Remove comments (if any)
         if func is not None:
@@ -258,45 +268,49 @@ class FlavorParser:
                     f = float(item[1:]) / 60
                 if item[0] == "E":
                     e = float(item[1:])
-            if self._is_absolute_positioning and ((x is not None and x < 0) or (y is not None and y < 0)):
-                self._center_is_zero = True
-            params = self._position(x, y, z, f, e)
+            params = PositionOptional(x, y, z, f, e)
             return func(position, params, path)
         return position
 
-    def processTCode(self, T, line, position, path):
+    def processTCode(self, T: int, line: str, position: Position, path: List[List[Union[float, int]]]) -> Position:
         self._extruder_number = T
         if self._extruder_number + 1 > len(position.e):
             self._extrusion_length_offset.extend([0] * (self._extruder_number - len(position.e) + 1))
             position.e.extend([0] * (self._extruder_number - len(position.e) + 1))
         return position
 
-    def processMCode(self, M, line, position, path):
+    def processMCode(self, M: int, line: str, position: Position, path: List[List[Union[float, int]]]) -> Position:
         pass
 
     _type_keyword = ";TYPE:"
     _layer_keyword = ";LAYER:"
 
     ##  For showing correct x, y offsets for each extruder
-    def _extruderOffsets(self):
+    def _extruderOffsets(self) -> Dict[int, List[float]]:
         result = {}
-        for extruder in ExtruderManager.getInstance().getExtruderStacks():
+        for extruder in ExtruderManager.getInstance().getActiveExtruderStacks():
             result[int(extruder.getMetaData().get("position", "0"))] = [
                 extruder.getProperty("machine_nozzle_offset_x", "value"),
                 extruder.getProperty("machine_nozzle_offset_y", "value")]
         return result
 
-    def processGCodeStream(self, stream):
+    #
+    # CURA-6643
+    # This function needs the filename so it can be set to the SceneNode. Otherwise, if you load a GCode file and press
+    # F5, that gcode SceneNode will be removed because it doesn't have a file to be reloaded from.
+    #
+    def processGCodeStream(self, stream: str, filename: str) -> Optional["CuraSceneNode"]:
         Logger.log("d", "Preparing to load GCode")
         self._cancelled = False
         # We obtain the filament diameter from the selected extruder to calculate line widths
-        global_stack = Application.getInstance().getGlobalContainerStack()
+        global_stack = CuraApplication.getInstance().getGlobalContainerStack()
+
+        if not global_stack:
+            return None
+
         self._filament_diameter = global_stack.extruders[str(self._extruder_number)].getProperty("material_diameter", "value")
 
         scene_node = CuraSceneNode()
-        # Override getBoundingBox function of the sceneNode, as this node should return a bounding box, but there is no
-        # real data to calculate it from.
-        scene_node.getBoundingBox = self._getNullBoundingBox
 
         gcode_list = []
         self._is_layers_in_file = False
@@ -322,16 +336,18 @@ class FlavorParser:
                                 lifetime=0,
                                 title = catalog.i18nc("@info:title", "G-code Details"))
 
+        assert(self._message is not None) # use for typing purposes
         self._message.setProgress(0)
         self._message.show()
 
         Logger.log("d", "Parsing Gcode...")
 
-        current_position = self._position(0, 0, 0, 0, [0])
-        current_path = []
+        current_position = Position(0, 0, 0, 0, [0])
+        current_path = [] #type: List[List[float]]
         min_layer_number = 0
         negative_layers = 0
         previous_layer = 0
+        self._previous_extrusion_value = 0.0
 
         for line in stream.split("\n"):
             if self._cancelled:
@@ -359,15 +375,21 @@ class FlavorParser:
                     self._layer_type = LayerPolygon.SupportType
                 elif type == "FILL":
                     self._layer_type = LayerPolygon.InfillType
+                elif type == "SUPPORT-INTERFACE":
+                    self._layer_type = LayerPolygon.SupportInterfaceType
+                elif type == "PRIME-TOWER":
+                    self._layer_type = LayerPolygon.PrimeTowerType
                 else:
                     Logger.log("w", "Encountered a unknown type (%s) while parsing g-code.", type)
 
-            # When the layer change is reached, the polygon is computed so we have just one layer per layer per extruder
+            # When the layer change is reached, the polygon is computed so we have just one layer per extruder
             if self._is_layers_in_file and line[:len(self._layer_keyword)] == self._layer_keyword:
                 try:
                     layer_number = int(line[len(self._layer_keyword):])
                     self._createPolygon(self._current_layer_thickness, current_path, self._extruder_offsets.get(self._extruder_number, [0, 0]))
                     current_path.clear()
+                    # Start the new layer at the end position of the last layer
+                    current_path.append([current_position.x, current_position.y, current_position.z, current_position.f, current_position.e[self._extruder_number], LayerPolygon.MoveCombingType])
 
                     # When using a raft, the raft layers are stored as layers < 0, it mimics the same behavior
                     # as in ProcessSlicedLayersJob
@@ -403,14 +425,20 @@ class FlavorParser:
             if line.startswith("T"):
                 T = self._getInt(line, "T")
                 if T is not None:
+                    self._extruders_seen.add(T)
                     self._createPolygon(self._current_layer_thickness, current_path, self._extruder_offsets.get(self._extruder_number, [0, 0]))
                     current_path.clear()
 
+                    # When changing tool, store the end point of the previous path, then process the code and finally
+                    # add another point with the new position of the head.
+                    current_path.append([current_position.x, current_position.y, current_position.z, current_position.f, current_position.e[self._extruder_number], LayerPolygon.MoveCombingType])
                     current_position = self.processTCode(T, line, current_position, current_path)
+                    current_path.append([current_position.x, current_position.y, current_position.z, current_position.f, current_position.e[self._extruder_number], LayerPolygon.MoveCombingType])
 
             if line.startswith("M"):
                 M = self._getInt(line, "M")
-                self.processMCode(M, line, current_position, current_path)
+                if M is not None:
+                    self.processMCode(M, line, current_position, current_path)
 
         # "Flush" leftovers. Last layer paths are still stored
         if len(current_path) > 1:
@@ -433,13 +461,14 @@ class FlavorParser:
         scene_node.addDecorator(decorator)
 
         gcode_list_decorator = GCodeListDecorator()
+        gcode_list_decorator.setGcodeFileName(filename)
         gcode_list_decorator.setGCodeList(gcode_list)
         scene_node.addDecorator(gcode_list_decorator)
 
         # gcode_dict stores gcode_lists for a number of build plates.
-        active_build_plate_id = Application.getInstance().getMultiBuildPlateModel().activeBuildPlate
+        active_build_plate_id = CuraApplication.getInstance().getMultiBuildPlateModel().activeBuildPlate
         gcode_dict = {active_build_plate_id: gcode_list}
-        Application.getInstance().getController().getScene().gcode_dict = gcode_dict
+        CuraApplication.getInstance().getController().getScene().gcode_dict = gcode_dict #type: ignore #Because gcode_dict is generated dynamically.
 
         Logger.log("d", "Finished parsing Gcode")
         self._message.hide()
@@ -447,16 +476,14 @@ class FlavorParser:
         if self._layer_number == 0:
             Logger.log("w", "File doesn't contain any valid layers")
 
-        settings = Application.getInstance().getGlobalContainerStack()
-        machine_width = settings.getProperty("machine_width", "value")
-        machine_depth = settings.getProperty("machine_depth", "value")
-
-        if not self._center_is_zero:
+        if not global_stack.getProperty("machine_center_is_zero", "value"):
+            machine_width = global_stack.getProperty("machine_width", "value")
+            machine_depth = global_stack.getProperty("machine_depth", "value")
             scene_node.setPosition(Vector(-machine_width / 2, 0, machine_depth / 2))
 
         Logger.log("d", "GCode loading finished")
 
-        if Preferences.getInstance().getValue("gcodereader/show_caution"):
+        if CuraApplication.getInstance().getPreferences().getValue("gcodereader/show_caution"):
             caution_message = Message(catalog.i18nc(
                 "@info:generic",
                 "Make sure the g-code is suitable for your printer and printer configuration before sending the file to it. The g-code representation may not be accurate."),
@@ -465,7 +492,7 @@ class FlavorParser:
             caution_message.show()
 
         # The "save/print" button's state is bound to the backend state.
-        backend = Application.getInstance().getBackend()
+        backend = CuraApplication.getInstance().getBackend()
         backend.backendStateChange.emit(Backend.BackendState.Disabled)
 
         return scene_node
