@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import configparser  # For reading the legacy profile INI files.
@@ -6,14 +6,14 @@ import io
 import json  # For reading the Dictionary of Doom.
 import math  # For mathematical operations included in the Dictionary of Doom.
 import os.path  # For concatenating the path to the plugin and the relative path to the Dictionary of Doom.
+from typing import Dict
 
 from UM.Application import Application  # To get the machine manager to create the new profile in.
 from UM.Logger import Logger  # Logging errors.
 from UM.PluginRegistry import PluginRegistry  # For getting the path to this plugin's directory.
 from UM.Settings.ContainerRegistry import ContainerRegistry #To create unique profile IDs.
 from UM.Settings.InstanceContainer import InstanceContainer  # The new profile to make.
-from cura.ProfileReader import ProfileReader  # The plug-in type to implement.
-from cura.Settings.ExtruderManager import ExtruderManager #To get the current extruder definition.
+from cura.ReaderWriters.ProfileReader import ProfileReader  # The plug-in type to implement.
 
 
 ##  A plugin that reads profile data from legacy Cura versions.
@@ -34,10 +34,11 @@ class LegacyProfileReader(ProfileReader):
     #   \param json The JSON file to load the default setting values from. This
     #   should not be a URL but a pre-loaded JSON handle.
     #   \return A dictionary of the default values of the legacy Cura version.
-    def prepareDefaults(self, json):
+    def prepareDefaults(self, json: Dict[str, Dict[str, str]]) -> Dict[str, str]:
         defaults = {}
-        for key in json["defaults"]:  # We have to copy over all defaults from the JSON handle to a normal dict.
-            defaults[key] = json["defaults"][key]
+        if "defaults" in json:
+            for key in json["defaults"]:  # We have to copy over all defaults from the JSON handle to a normal dict.
+                defaults[key] = json["defaults"][key]
         return defaults
 
     ##  Prepares the local variables that can be used in evaluation of computing
@@ -81,11 +82,10 @@ class LegacyProfileReader(ProfileReader):
         Logger.log("i", "Importing legacy profile from file " + file_name + ".")
         container_registry = ContainerRegistry.getInstance()
         profile_id = container_registry.uniqueName("Imported Legacy Profile")
-        profile = InstanceContainer(profile_id)  # Create an empty profile.
 
-        parser = configparser.ConfigParser(interpolation = None)
+        input_parser = configparser.ConfigParser(interpolation = None)
         try:
-            parser.read([file_name])  # Parse the INI file.
+            input_parser.read([file_name])  # Parse the INI file.
         except Exception as e:
             Logger.log("e", "Unable to open legacy profile %s: %s", file_name, str(e))
             return None
@@ -93,7 +93,7 @@ class LegacyProfileReader(ProfileReader):
         # Legacy Cura saved the profile under the section "profile_N" where N is the ID of a machine, except when you export in which case it saves it in the section "profile".
         # Since importing multiple machine profiles is out of scope, just import the first section we find.
         section = ""
-        for found_section in parser.sections():
+        for found_section in input_parser.sections():
             if found_section.startswith("profile"):
                 section = found_section
                 break
@@ -101,7 +101,7 @@ class LegacyProfileReader(ProfileReader):
             return None
 
         try:
-            with open(os.path.join(PluginRegistry.getInstance().getPluginPath("LegacyProfileReader"), "DictionaryOfDoom.json"), "r", -1, "utf-8") as f:
+            with open(os.path.join(PluginRegistry.getInstance().getPluginPath("LegacyProfileReader"), "DictionaryOfDoom.json"), "r", encoding = "utf-8") as f:
                 dict_of_doom = json.load(f)  # Parse the Dictionary of Doom.
         except IOError as e:
             Logger.log("e", "Could not open DictionaryOfDoom.json for reading: %s", str(e))
@@ -111,15 +111,13 @@ class LegacyProfileReader(ProfileReader):
             return None
 
         defaults = self.prepareDefaults(dict_of_doom)
-        legacy_settings = self.prepareLocals(parser, section, defaults) #Gets the settings from the legacy profile.
+        legacy_settings = self.prepareLocals(input_parser, section, defaults) #Gets the settings from the legacy profile.
 
-        #Check the target version in the Dictionary of Doom with this application version.
-        if "target_version" not in dict_of_doom:
-            Logger.log("e", "Dictionary of Doom has no target version. Is it the correct JSON file?")
-            return None
-        if InstanceContainer.Version != dict_of_doom["target_version"]:
-            Logger.log("e", "Dictionary of Doom of legacy profile reader (version %s) is not in sync with the current instance container version (version %s)!", dict_of_doom["target_version"], str(InstanceContainer.Version))
-            return None
+        # Serialised format into version 4.5. Do NOT upgrade this, let the version upgrader handle it.
+        output_parser = configparser.ConfigParser(interpolation = None)
+        output_parser.add_section("general")
+        output_parser.add_section("metadata")
+        output_parser.add_section("values")
 
         if "translation" not in dict_of_doom:
             Logger.log("e", "Dictionary of Doom has no translation. Is it the correct JSON file?")
@@ -128,7 +126,7 @@ class LegacyProfileReader(ProfileReader):
         quality_definition = current_printer_definition.getMetaDataEntry("quality_definition")
         if not quality_definition:
             quality_definition = current_printer_definition.getId()
-        profile.setDefinition(quality_definition)
+        output_parser["general"]["definition"] = quality_definition
         for new_setting in dict_of_doom["translation"]:  # Evaluate all new settings that would get a value from the translations.
             old_setting_expression = dict_of_doom["translation"][new_setting]
             compiled = compile(old_setting_expression, new_setting, "eval")
@@ -141,37 +139,34 @@ class LegacyProfileReader(ProfileReader):
             definitions = current_printer_definition.findDefinitions(key = new_setting)
             if definitions:
                 if new_value != value_using_defaults and definitions[0].default_value != new_value:  # Not equal to the default in the new Cura OR the default in the legacy Cura.
-                    profile.setProperty(new_setting, "value", new_value)  # Store the setting in the profile!
+                    output_parser["values"][new_setting] = str(new_value) # Store the setting in the profile!
 
-        if len(profile.getAllKeys()) == 0:
+        if len(output_parser["values"]) == 0:
             Logger.log("i", "A legacy profile was imported but everything evaluates to the defaults, creating an empty profile.")
 
-        profile.addMetaDataEntry("type", "profile")
-        # don't know what quality_type it is based on, so use "normal" by default
-        profile.addMetaDataEntry("quality_type", "normal")
-        profile.setName(profile_id)
-        profile.setDirty(True)
+        output_parser["general"]["version"] = "4"
+        output_parser["general"]["name"] = profile_id
+        output_parser["metadata"]["type"] = "quality_changes"
+        output_parser["metadata"]["quality_type"] = "normal" # Don't know what quality_type it is based on, so use "normal" by default.
+        output_parser["metadata"]["position"] = "0" # We only support single extrusion.
+        output_parser["metadata"]["setting_version"] = "5" # What the dictionary of doom is made for.
 
-        #Serialise and deserialise in order to perform the version upgrade.
-        parser = configparser.ConfigParser(interpolation=None)
-        data = profile.serialize()
-        parser.read_string(data)
-        parser["general"]["version"] = "1"
-        if parser.has_section("values"):
-            parser["settings"] = parser["values"]
-            del parser["values"]
+        # Serialise in order to perform the version upgrade.
         stream = io.StringIO()
-        parser.write(stream)
+        output_parser.write(stream)
         data = stream.getvalue()
-        profile.deserialize(data)
 
-        # The definition can get reset to fdmprinter during the deserialization's upgrade. Here we set the definition
-        # again.
-        profile.setDefinition(quality_definition)
+        profile = InstanceContainer(profile_id)
+        profile.deserialize(data) # Also performs the version upgrade.
+        profile.setDirty(True)
 
         #We need to return one extruder stack and one global stack.
         global_container_id = container_registry.uniqueName("Global Imported Legacy Profile")
+        # We duplicate the extruder profile into the global stack.
+        # This may introduce some settings that are global in the extruder stack and some settings that are per-extruder in the global stack.
+        # We don't care about that. The engine will ignore them anyway.
         global_profile = profile.duplicate(new_id = global_container_id, new_name = profile_id) #Needs to have the same name as the extruder profile.
+        del global_profile.getMetaData()["position"] # Has no position because it's global.
         global_profile.setDirty(True)
 
         profile_definition = "fdmprinter"
