@@ -3,8 +3,6 @@
 
 from . import MachineSettingsAction
 
-from UM.i18n import i18nCatalog
-catalog = i18nCatalog("cura")
 
 def getMetaData():
     return {}
