@@ -1,43 +1,18 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import pytest #This module contains automated tests.
 import unittest.mock #For the mocking and monkeypatching functionality.
-import copy
 
-import cura.CuraApplication
+import cura.Settings.CuraContainerStack #To get the list of container types.
 import UM.Settings.ContainerRegistry #To create empty instance containers.
 import UM.Settings.ContainerStack #To set the container registry the container stacks use.
 from UM.Settings.DefinitionContainer import DefinitionContainer #To check against the class of DefinitionContainer.
 from UM.Settings.InstanceContainer import InstanceContainer #To check against the class of InstanceContainer.
-import cura.Settings.ExtruderStack #The module we're testing.
+from cura.Settings import Exceptions
 from cura.Settings.Exceptions import InvalidContainerError, InvalidOperationError #To check whether the correct exceptions are raised.
-
 from cura.Settings.ExtruderManager import ExtruderManager
-from UM.Settings.ContainerRegistry import ContainerRegistry
-from cura.Settings.GlobalStack import GlobalStack
-
-##  Fake container registry that always provides all containers you ask of.
-@pytest.yield_fixture()
-def container_registry():
-    registry = unittest.mock.MagicMock()
-    registry.return_value = unittest.mock.NonCallableMagicMock()
-    registry.findInstanceContainers = lambda *args, registry = registry, **kwargs: [registry.return_value]
-    registry.findDefinitionContainers = lambda *args, registry = registry, **kwargs: [registry.return_value]
-
-    UM.Settings.ContainerRegistry.ContainerRegistry._ContainerRegistry__instance = registry
-    UM.Settings.ContainerStack._containerRegistry = registry
-
-    yield registry
-
-    UM.Settings.ContainerRegistry.ContainerRegistry._ContainerRegistry__instance = None
-    UM.Settings.ContainerStack._containerRegistry = None
-
-##  An empty extruder stack to test with.
-@pytest.fixture()
-def extruder_stack() -> cura.Settings.ExtruderStack.ExtruderStack:
-    creteEmptyContainers()
-    return cura.Settings.ExtruderStack.ExtruderStack("TestStack")
+from cura.Settings.cura_empty_instance_containers import empty_container
 
 ##  Gets an instance container with a specified container type.
 #
@@ -45,34 +20,9 @@ def extruder_stack() -> cura.Settings.ExtruderStack.ExtruderStack:
 #   \return An instance container instance.
 def getInstanceContainer(container_type) -> InstanceContainer:
     container = InstanceContainer(container_id = "InstanceContainer")
-    container.addMetaDataEntry("type", container_type)
+    container.setMetaDataEntry("type", container_type)
     return container
 
-def creteEmptyContainers():
-    empty_container = ContainerRegistry.getInstance().getEmptyInstanceContainer()
-    empty_variant_container = copy.deepcopy(empty_container)
-    empty_variant_container.setMetaDataEntry("id", "empty_variant")
-    empty_variant_container.addMetaDataEntry("type", "variant")
-    ContainerRegistry.getInstance().addContainer(empty_variant_container)
-
-    empty_material_container = copy.deepcopy(empty_container)
-    empty_material_container.setMetaDataEntry("id", "empty_material")
-    empty_material_container.addMetaDataEntry("type", "material")
-    ContainerRegistry.getInstance().addContainer(empty_material_container)
-
-    empty_quality_container = copy.deepcopy(empty_container)
-    empty_quality_container.setMetaDataEntry("id", "empty_quality")
-    empty_quality_container.setName("Not Supported")
-    empty_quality_container.addMetaDataEntry("quality_type", "not_supported")
-    empty_quality_container.addMetaDataEntry("type", "quality")
-    empty_quality_container.addMetaDataEntry("supported", False)
-    ContainerRegistry.getInstance().addContainer(empty_quality_container)
-
-    empty_quality_changes_container = copy.deepcopy(empty_container)
-    empty_quality_changes_container.setMetaDataEntry("id", "empty_quality_changes")
-    empty_quality_changes_container.addMetaDataEntry("type", "quality_changes")
-    ContainerRegistry.getInstance().addContainer(empty_quality_changes_container)
-
 class DefinitionContainerSubClass(DefinitionContainer):
     def __init__(self):
         super().__init__(container_id = "SubDefinitionContainer")
@@ -80,7 +30,7 @@ class DefinitionContainerSubClass(DefinitionContainer):
 class InstanceContainerSubClass(InstanceContainer):
     def __init__(self, container_type):
         super().__init__(container_id = "SubInstanceContainer")
-        self.addMetaDataEntry("type", container_type)
+        self.setMetaDataEntry("type", container_type)
 
 #############################START OF TEST CASES################################
 
@@ -179,12 +129,30 @@ def test_constrainVariantInvalid(container, extruder_stack):
 def test_constrainVariantValid(container, extruder_stack):
     extruder_stack.variant = container #Should not give an error.
 
+#Tests setting definition changes profiles to invalid containers.
+@pytest.mark.parametrize("container", [
+    getInstanceContainer(container_type = "wrong container type"),
+    getInstanceContainer(container_type = "material"), #Existing, but still wrong type.
+    DefinitionContainer(container_id = "wrong class")
+])
+def test_constrainDefinitionChangesInvalid(container, global_stack):
+    with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
+        global_stack.definitionChanges = container
+
+#Test setting definition changes profiles.
+@pytest.mark.parametrize("container", [
+    getInstanceContainer(container_type = "definition_changes"),
+    InstanceContainerSubClass(container_type = "definition_changes")
+])
+def test_constrainDefinitionChangesValid(container, global_stack):
+    global_stack.definitionChanges = container #Should not give an error.
+
 #Tests setting definitions to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong class"),
     getInstanceContainer(container_type = "material"), #Existing, but still wrong class.
 ])
-def test_constrainVariantInvalid(container, extruder_stack):
+def test_constrainDefinitionInvalid(container, extruder_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         extruder_stack.definition = container
 
@@ -196,23 +164,22 @@ def test_constrainVariantInvalid(container, extruder_stack):
 def test_constrainDefinitionValid(container, extruder_stack):
     extruder_stack.definition = container #Should not give an error.
 
-##  Tests whether deserialising completes the missing containers with empty
-#   ones.
-@pytest.mark.skip #The test currently fails because the definition container doesn't have a category, which is wrong but we don't have time to refactor that right now.
-def test_deserializeCompletesEmptyContainers(extruder_stack: cura.Settings.ExtruderStack):
-    extruder_stack._containers = [DefinitionContainer(container_id = "definition")] #Set the internal state of this stack manually.
+##  Tests whether deserialising completes the missing containers with empty ones.
+def test_deserializeCompletesEmptyContainers(extruder_stack):
+    extruder_stack._containers = [DefinitionContainer(container_id = "definition"), extruder_stack.definitionChanges] #Set the internal state of this stack manually.
 
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         extruder_stack.deserialize("")
 
     assert len(extruder_stack.getContainers()) == len(cura.Settings.CuraContainerStack._ContainerIndexes.IndexTypeMap) #Needs a slot for every type.
     for container_type_index in cura.Settings.CuraContainerStack._ContainerIndexes.IndexTypeMap:
-        if container_type_index == cura.Settings.CuraContainerStack._ContainerIndexes.Definition: #We're not checking the definition.
+        if container_type_index in \
+                (cura.Settings.CuraContainerStack._ContainerIndexes.Definition,
+                 cura.Settings.CuraContainerStack._ContainerIndexes.DefinitionChanges):  # We're not checking the definition or definition_changes
             continue
-        assert extruder_stack.getContainer(container_type_index).getId() == "empty" #All others need to be empty.
+        assert extruder_stack.getContainer(container_type_index) == empty_container #All others need to be empty.
 
-##  Tests whether an instance container with the wrong type gets removed when
-#   deserialising.
+##  Tests whether an instance container with the wrong type gets removed when deserialising.
 def test_deserializeRemovesWrongInstanceContainer(extruder_stack):
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = getInstanceContainer(container_type = "wrong type")
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -222,8 +189,7 @@ def test_deserializeRemovesWrongInstanceContainer(extruder_stack):
 
     assert extruder_stack.quality == extruder_stack._empty_instance_container #Replaced with empty.
 
-##  Tests whether a container with the wrong class gets removed when
-#   deserialising.
+##  Tests whether a container with the wrong class gets removed when deserialising.
 def test_deserializeRemovesWrongContainerClass(extruder_stack):
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = DefinitionContainer(container_id = "wrong class")
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -233,8 +199,7 @@ def test_deserializeRemovesWrongContainerClass(extruder_stack):
 
     assert extruder_stack.quality == extruder_stack._empty_instance_container #Replaced with empty.
 
-##  Tests whether an instance container in the definition spot results in an
-#   error.
+##  Tests whether an instance container in the definition spot results in an error.
 def test_deserializeWrongDefinitionClass(extruder_stack):
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = getInstanceContainer(container_type = "definition") #Correct type but wrong class.
 
@@ -242,8 +207,7 @@ def test_deserializeWrongDefinitionClass(extruder_stack):
         with pytest.raises(UM.Settings.ContainerStack.InvalidContainerStackError): #Must raise an error that there is no definition container.
             extruder_stack.deserialize("")
 
-##  Tests whether an instance container with the wrong type is moved into the
-#   correct slot by deserialising.
+##  Tests whether an instance container with the wrong type is moved into the correct slot by deserialising.
 def test_deserializeMoveInstanceContainer(extruder_stack):
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = getInstanceContainer(container_type = "material") #Not in the correct spot.
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -251,26 +215,21 @@ def test_deserializeMoveInstanceContainer(extruder_stack):
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         extruder_stack.deserialize("")
 
-    assert extruder_stack.quality.getId() == "empty"
-    assert extruder_stack.material.getId() != "empty"
-from UM.Settings.Validator import Validator
-##  Tests whether a definition container in the wrong spot is moved into the
-#   correct spot by deserialising.
-@pytest.mark.skip #The test currently fails because the definition container doesn't have a category, which is wrong but we don't have time to refactor that right now.
+    assert extruder_stack.quality == empty_container
+    assert extruder_stack.material != empty_container
+
+##  Tests whether a definition container in the wrong spot is moved into the correct spot by deserialising.
 def test_deserializeMoveDefinitionContainer(extruder_stack):
     extruder_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Material] = DefinitionContainer(container_id = "some definition") #Not in the correct spot.
 
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         extruder_stack.deserialize("")
 
-    assert extruder_stack.material.getId() == "empty"
-    assert extruder_stack.definition.getId() != "empty"
+    assert extruder_stack.material == empty_container
+    assert extruder_stack.definition != empty_container
 
-    UM.Settings.ContainerStack._containerRegistry = None
-
-##  Tests whether getProperty properly applies the stack-like behaviour on its
-#   containers.
-def test_getPropertyFallThrough(extruder_stack):
+##  Tests whether getProperty properly applies the stack-like behaviour on its containers.
+def test_getPropertyFallThrough(global_stack, extruder_stack):
     # ExtruderStack.setNextStack calls registerExtruder for backward compatibility, but we do not need a complete extruder manager
     ExtruderManager._ExtruderManager__instance = unittest.mock.MagicMock()
 
@@ -300,8 +259,7 @@ def test_getPropertyFallThrough(extruder_stack):
     with unittest.mock.patch("cura.Settings.CuraContainerStack.DefinitionContainer", unittest.mock.MagicMock): #To guard against the type checking.
         extruder_stack.definition = mock_layer_heights[container_indices.Definition] #There's a layer height in here!
 
-    stack = GlobalStack("PyTest GlobalStack")
-    extruder_stack.setNextStack(stack)
+    extruder_stack.setNextStack(global_stack)
 
     assert extruder_stack.getProperty("layer_height", "value") == container_indices.Definition
     extruder_stack.variant = mock_layer_heights[container_indices.Variant]
@@ -340,26 +298,31 @@ def test_setPropertyUser(key, property, value, extruder_stack):
 
     extruder_stack.setProperty(key, property, value) #The actual test.
 
-    extruder_stack.userChanges.setProperty.assert_called_once_with(key, property, value) #Make sure that the user container gets a setProperty call.
+    extruder_stack.userChanges.setProperty.assert_called_once_with(key, property, value, None, False) #Make sure that the user container gets a setProperty call.
 
-##  Tests setting properties on specific containers on the global stack.
-@pytest.mark.parametrize("target_container,    stack_variable", [
-                        ("user",               "userChanges"),
-                        ("quality_changes",    "qualityChanges"),
-                        ("quality",            "quality"),
-                        ("material",           "material"),
-                        ("variant",            "variant")
-])
-def test_setPropertyOtherContainers(target_container, stack_variable, extruder_stack):
-    #Other parameters that don't need to be varied.
-    key = "layer_height"
-    property = "value"
-    value = 0.1337
-    #A mock container in the right spot.
-    container = unittest.mock.MagicMock()
-    container.getMetaDataEntry = unittest.mock.MagicMock(return_value = target_container)
-    setattr(extruder_stack, stack_variable, container) #For instance, set global_stack.qualityChanges = container.
 
-    extruder_stack.setProperty(key, property, value, target_container = target_container) #The actual test.
+def test_setEnabled(extruder_stack):
+    extruder_stack.setEnabled(True)
+    assert extruder_stack.isEnabled
+    extruder_stack.setEnabled(False)
+    assert not extruder_stack.isEnabled
 
-    getattr(extruder_stack, stack_variable).setProperty.assert_called_once_with(key, property, value) #Make sure that the proper container gets a setProperty call.
+
+def test_getPropertyWithoutGlobal(extruder_stack):
+    assert extruder_stack.getNextStack() is None
+
+    with pytest.raises(Exceptions.NoGlobalStackError):
+        extruder_stack.getProperty("whatever", "value")
+
+
+def test_getMachineDefinitionWithoutGlobal(extruder_stack):
+    assert extruder_stack.getNextStack() is None
+
+    with pytest.raises(Exceptions.NoGlobalStackError):
+        extruder_stack._getMachineDefinition()
+
+def test_getMachineDefinition(extruder_stack):
+    mocked_next_stack = unittest.mock.MagicMock()
+    mocked_next_stack._getMachineDefinition = unittest.mock.MagicMock(return_value = "ZOMG")
+    extruder_stack.getNextStack = unittest.mock.MagicMock(return_value = mocked_next_stack)
+    assert extruder_stack._getMachineDefinition() == "ZOMG"
\ No newline at end of file
