@@ -1,43 +1,20 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Cura is released under the terms of the LGPLv3 or higher.
 
 import pytest #This module contains unit tests.
 import unittest.mock #To monkeypatch some mocks in place of dependencies.
-import copy
 
-import cura.CuraApplication
-import cura.Settings.GlobalStack #The module we're testing.
 import cura.Settings.CuraContainerStack #To get the list of container types.
-from cura.Settings.Exceptions import TooManyExtrudersError, InvalidContainerError, InvalidOperationError #To test raising these errors.
+from cura.Settings.Exceptions import InvalidContainerError, InvalidOperationError #To test raising these errors.
 from UM.Settings.DefinitionContainer import DefinitionContainer #To test against the class DefinitionContainer.
 from UM.Settings.InstanceContainer import InstanceContainer #To test against the class InstanceContainer.
 from UM.Settings.SettingInstance import InstanceState
 import UM.Settings.ContainerRegistry
 import UM.Settings.ContainerStack
 import UM.Settings.SettingDefinition #To add settings to the definition.
-from UM.Settings.ContainerRegistry import ContainerRegistry
 
-##  Fake container registry that always provides all containers you ask of.
-@pytest.yield_fixture()
-def container_registry():
-    registry = unittest.mock.MagicMock()
-    registry.return_value = unittest.mock.NonCallableMagicMock()
-    registry.findInstanceContainers = lambda *args, registry = registry, **kwargs: [registry.return_value]
-    registry.findDefinitionContainers = lambda *args, registry = registry, **kwargs: [registry.return_value]
+from cura.Settings.cura_empty_instance_containers import empty_container
 
-    UM.Settings.ContainerRegistry.ContainerRegistry._ContainerRegistry__instance = registry
-    UM.Settings.ContainerStack._containerRegistry = registry
-
-    yield registry
-
-    UM.Settings.ContainerRegistry.ContainerRegistry._ContainerRegistry__instance = None
-    UM.Settings.ContainerStack._containerRegistry = None
-
-#An empty global stack to test with.
-@pytest.fixture()
-def global_stack() -> cura.Settings.GlobalStack.GlobalStack:
-    creteEmptyContainers()
-    return cura.Settings.GlobalStack.GlobalStack("TestStack")
 
 ##  Gets an instance container with a specified container type.
 #
@@ -45,50 +22,30 @@ def global_stack() -> cura.Settings.GlobalStack.GlobalStack:
 #   \return An instance container instance.
 def getInstanceContainer(container_type) -> InstanceContainer:
     container = InstanceContainer(container_id = "InstanceContainer")
-    container.addMetaDataEntry("type", container_type)
+    container.setMetaDataEntry("type", container_type)
     return container
 
-def creteEmptyContainers():
-    empty_container = ContainerRegistry.getInstance().getEmptyInstanceContainer()
-    empty_variant_container = copy.deepcopy(empty_container)
-    empty_variant_container.setMetaDataEntry("id", "empty_variant")
-    empty_variant_container.addMetaDataEntry("type", "variant")
-    ContainerRegistry.getInstance().addContainer(empty_variant_container)
-
-    empty_material_container = copy.deepcopy(empty_container)
-    empty_material_container.setMetaDataEntry("id", "empty_material")
-    empty_material_container.addMetaDataEntry("type", "material")
-    ContainerRegistry.getInstance().addContainer(empty_material_container)
-
-    empty_quality_container = copy.deepcopy(empty_container)
-    empty_quality_container.setMetaDataEntry("id", "empty_quality")
-    empty_quality_container.setName("Not Supported")
-    empty_quality_container.addMetaDataEntry("quality_type", "not_supported")
-    empty_quality_container.addMetaDataEntry("type", "quality")
-    empty_quality_container.addMetaDataEntry("supported", False)
-    ContainerRegistry.getInstance().addContainer(empty_quality_container)
-
-    empty_quality_changes_container = copy.deepcopy(empty_container)
-    empty_quality_changes_container.setMetaDataEntry("id", "empty_quality_changes")
-    empty_quality_changes_container.addMetaDataEntry("type", "quality_changes")
-    ContainerRegistry.getInstance().addContainer(empty_quality_changes_container)
 
 class DefinitionContainerSubClass(DefinitionContainer):
     def __init__(self):
         super().__init__(container_id = "SubDefinitionContainer")
 
+
 class InstanceContainerSubClass(InstanceContainer):
     def __init__(self, container_type):
         super().__init__(container_id = "SubInstanceContainer")
-        self.addMetaDataEntry("type", container_type)
+        self.setMetaDataEntry("type", container_type)
+
 
 #############################START OF TEST CASES################################
 
+
 ##  Tests whether adding a container is properly forbidden.
 def test_addContainer(global_stack):
     with pytest.raises(InvalidOperationError):
         global_stack.addContainer(unittest.mock.MagicMock())
 
+
 ##  Tests adding extruders to the global stack.
 def test_addExtruder(global_stack):
     mock_definition = unittest.mock.MagicMock()
@@ -116,6 +73,7 @@ def test_addExtruder(global_stack):
     #         global_stack.addExtruder(unittest.mock.MagicMock())
     assert len(global_stack.extruders) == 2 #Didn't add the faulty extruder.
 
+
 #Tests setting user changes profiles to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -126,6 +84,7 @@ def test_constrainUserChangesInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.userChanges = container
 
+
 #Tests setting user changes profiles.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "user"),
@@ -134,6 +93,7 @@ def test_constrainUserChangesInvalid(container, global_stack):
 def test_constrainUserChangesValid(container, global_stack):
     global_stack.userChanges = container #Should not give an error.
 
+
 #Tests setting quality changes profiles to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -144,6 +104,7 @@ def test_constrainQualityChangesInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.qualityChanges = container
 
+
 #Test setting quality changes profiles.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "quality_changes"),
@@ -152,6 +113,7 @@ def test_constrainQualityChangesInvalid(container, global_stack):
 def test_constrainQualityChangesValid(container, global_stack):
     global_stack.qualityChanges = container #Should not give an error.
 
+
 #Tests setting quality profiles to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -162,6 +124,7 @@ def test_constrainQualityInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.quality = container
 
+
 #Test setting quality profiles.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "quality"),
@@ -170,6 +133,7 @@ def test_constrainQualityInvalid(container, global_stack):
 def test_constrainQualityValid(container, global_stack):
     global_stack.quality = container #Should not give an error.
 
+
 #Tests setting materials to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -180,6 +144,7 @@ def test_constrainMaterialInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.material = container
 
+
 #Test setting materials.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "material"),
@@ -188,6 +153,7 @@ def test_constrainMaterialInvalid(container, global_stack):
 def test_constrainMaterialValid(container, global_stack):
     global_stack.material = container #Should not give an error.
 
+
 #Tests setting variants to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -198,6 +164,7 @@ def test_constrainVariantInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.variant = container
 
+
 #Test setting variants.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "variant"),
@@ -206,6 +173,7 @@ def test_constrainVariantInvalid(container, global_stack):
 def test_constrainVariantValid(container, global_stack):
     global_stack.variant = container #Should not give an error.
 
+
 #Tests setting definition changes profiles to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong container type"),
@@ -216,6 +184,7 @@ def test_constrainDefinitionChangesInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.definitionChanges = container
 
+
 #Test setting definition changes profiles.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "definition_changes"),
@@ -224,15 +193,17 @@ def test_constrainDefinitionChangesInvalid(container, global_stack):
 def test_constrainDefinitionChangesValid(container, global_stack):
     global_stack.definitionChanges = container #Should not give an error.
 
+
 #Tests setting definitions to invalid containers.
 @pytest.mark.parametrize("container", [
     getInstanceContainer(container_type = "wrong class"),
     getInstanceContainer(container_type = "material"), #Existing, but still wrong class.
 ])
-def test_constrainVariantInvalid(container, global_stack):
+def test_constrainDefinitionInvalid(container, global_stack):
     with pytest.raises(InvalidContainerError): #Invalid container, should raise an error.
         global_stack.definition = container
 
+
 #Test setting definitions.
 @pytest.mark.parametrize("container", [
     DefinitionContainer(container_id = "DefinitionContainer"),
@@ -241,23 +212,24 @@ def test_constrainVariantInvalid(container, global_stack):
 def test_constrainDefinitionValid(container, global_stack):
     global_stack.definition = container #Should not give an error.
 
-##  Tests whether deserialising completes the missing containers with empty
-#   ones.
-@pytest.mark.skip #The test currently fails because the definition container doesn't have a category, which is wrong but we don't have time to refactor that right now.
-def test_deserializeCompletesEmptyContainers(global_stack: cura.Settings.GlobalStack):
-    global_stack._containers = [DefinitionContainer(container_id = "definition")] #Set the internal state of this stack manually.
+
+##  Tests whether deserialising completes the missing containers with empty ones. The initial containers are just the
+#   definition and the definition_changes (that cannot be empty after CURA-5281)
+def test_deserializeCompletesEmptyContainers(global_stack):
+    global_stack._containers = [DefinitionContainer(container_id = "definition"), global_stack.definitionChanges] #Set the internal state of this stack manually.
 
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         global_stack.deserialize("")
 
     assert len(global_stack.getContainers()) == len(cura.Settings.CuraContainerStack._ContainerIndexes.IndexTypeMap) #Needs a slot for every type.
     for container_type_index in cura.Settings.CuraContainerStack._ContainerIndexes.IndexTypeMap:
-        if container_type_index == cura.Settings.CuraContainerStack._ContainerIndexes.Definition: #We're not checking the definition.
+        if container_type_index in \
+                (cura.Settings.CuraContainerStack._ContainerIndexes.Definition, cura.Settings.CuraContainerStack._ContainerIndexes.DefinitionChanges): #We're not checking the definition or definition_changes
             continue
-        assert global_stack.getContainer(container_type_index).getId() == "empty" #All others need to be empty.
+        assert global_stack.getContainer(container_type_index) == empty_container #All others need to be empty.
 
-##  Tests whether an instance container with the wrong type gets removed when
-#   deserialising.
+
+##  Tests whether an instance container with the wrong type gets removed when deserialising.
 def test_deserializeRemovesWrongInstanceContainer(global_stack):
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = getInstanceContainer(container_type = "wrong type")
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -267,8 +239,8 @@ def test_deserializeRemovesWrongInstanceContainer(global_stack):
 
     assert global_stack.quality == global_stack._empty_instance_container #Replaced with empty.
 
-##  Tests whether a container with the wrong class gets removed when
-#   deserialising.
+
+##  Tests whether a container with the wrong class gets removed when deserialising.
 def test_deserializeRemovesWrongContainerClass(global_stack):
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = DefinitionContainer(container_id = "wrong class")
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -278,8 +250,8 @@ def test_deserializeRemovesWrongContainerClass(global_stack):
 
     assert global_stack.quality == global_stack._empty_instance_container #Replaced with empty.
 
-##  Tests whether an instance container in the definition spot results in an
-#   error.
+
+##  Tests whether an instance container in the definition spot results in an error.
 def test_deserializeWrongDefinitionClass(global_stack):
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = getInstanceContainer(container_type = "definition") #Correct type but wrong class.
 
@@ -287,8 +259,8 @@ def test_deserializeWrongDefinitionClass(global_stack):
         with pytest.raises(UM.Settings.ContainerStack.InvalidContainerStackError): #Must raise an error that there is no definition container.
             global_stack.deserialize("")
 
-##  Tests whether an instance container with the wrong type is moved into the
-#   correct slot by deserialising.
+
+##  Tests whether an instance container with the wrong type is moved into the correct slot by deserialising.
 def test_deserializeMoveInstanceContainer(global_stack):
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Quality] = getInstanceContainer(container_type = "material") #Not in the correct spot.
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Definition] = DefinitionContainer(container_id = "some definition")
@@ -296,25 +268,22 @@ def test_deserializeMoveInstanceContainer(global_stack):
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         global_stack.deserialize("")
 
-    assert global_stack.quality.getId() == "empty"
-    assert global_stack.material.getId() != "empty"
+    assert global_stack.quality == empty_container
+    assert global_stack.material != empty_container
 
-##  Tests whether a definition container in the wrong spot is moved into the
-#   correct spot by deserialising.
-@pytest.mark.skip #The test currently fails because the definition container doesn't have a category, which is wrong but we don't have time to refactor that right now.
+
+##  Tests whether a definition container in the wrong spot is moved into the correct spot by deserialising.
 def test_deserializeMoveDefinitionContainer(global_stack):
     global_stack._containers[cura.Settings.CuraContainerStack._ContainerIndexes.Material] = DefinitionContainer(container_id = "some definition") #Not in the correct spot.
 
     with unittest.mock.patch("UM.Settings.ContainerStack.ContainerStack.deserialize", unittest.mock.MagicMock()): #Prevent calling super().deserialize.
         global_stack.deserialize("")
 
-    assert global_stack.material.getId() == "empty"
-    assert global_stack.definition.getId() != "empty"
+    assert global_stack.material == empty_container
+    assert global_stack.definition != empty_container
 
-    UM.Settings.ContainerStack._containerRegistry = None
 
-##  Tests whether getProperty properly applies the stack-like behaviour on its
-#   containers.
+##  Tests whether getProperty properly applies the stack-like behaviour on its containers.
 def test_getPropertyFallThrough(global_stack):
     #A few instance container mocks to put in the stack.
     mock_layer_heights = {} #For each container type, a mock container that defines layer height to something unique.
@@ -356,6 +325,7 @@ def test_getPropertyFallThrough(global_stack):
     global_stack.userChanges = mock_layer_heights[container_indexes.UserChanges]
     assert global_stack.getProperty("layer_height", "value") == container_indexes.UserChanges
 
+
 ##  In definitions, test whether having no resolve allows us to find the value.
 def test_getPropertyNoResolveInDefinition(global_stack):
     value = unittest.mock.MagicMock() #Just sets the value for bed temperature.
@@ -365,8 +335,8 @@ def test_getPropertyNoResolveInDefinition(global_stack):
         global_stack.definition = value
     assert global_stack.getProperty("material_bed_temperature", "value") == 10 #No resolve, so fall through to value.
 
-##  In definitions, when the value is asked and there is a resolve function, it
-#   must get the resolve first.
+
+##  In definitions, when the value is asked and there is a resolve function, it must get the resolve first.
 def test_getPropertyResolveInDefinition(global_stack):
     resolve_and_value = unittest.mock.MagicMock() #Sets the resolve and value for bed temperature.
     resolve_and_value.getProperty = lambda key, property, context = None: (7.5 if property == "resolve" else 5) if (key == "material_bed_temperature" and property in ("resolve", "value")) else None #7.5 resolve, 5 value.
@@ -375,8 +345,8 @@ def test_getPropertyResolveInDefinition(global_stack):
         global_stack.definition = resolve_and_value
     assert global_stack.getProperty("material_bed_temperature", "value") == 7.5 #Resolve wins in the definition.
 
-##  In instance containers, when the value is asked and there is a resolve
-#   function, it must get the value first.
+
+##  In instance containers, when the value is asked and there is a resolve function, it must get the value first.
 def test_getPropertyResolveInInstance(global_stack):
     container_indices = cura.Settings.CuraContainerStack._ContainerIndexes
     instance_containers = {}
@@ -402,8 +372,8 @@ def test_getPropertyResolveInInstance(global_stack):
     global_stack.userChanges = instance_containers[container_indices.UserChanges]
     assert global_stack.getProperty("material_bed_temperature", "value") == 5
 
-##  Tests whether the value in instances gets evaluated before the resolve in
-#   definitions.
+
+##  Tests whether the value in instances gets evaluated before the resolve in definitions.
 def test_getPropertyInstancesBeforeResolve(global_stack):
     value = unittest.mock.MagicMock() #Sets just the value.
     value.getProperty = lambda key, property, context = None: (10 if property == "value" else (InstanceState.User if property != "limit_to_extruder" else "-1")) if key == "material_bed_temperature" else None
@@ -417,8 +387,8 @@ def test_getPropertyInstancesBeforeResolve(global_stack):
 
     assert global_stack.getProperty("material_bed_temperature", "value") == 10
 
-##  Tests whether the hasUserValue returns true for settings that are changed in
-#   the user-changes container.
+
+##  Tests whether the hasUserValue returns true for settings that are changed in the user-changes container.
 def test_hasUserValueUserChanges(global_stack):
     container = unittest.mock.MagicMock()
     container.getMetaDataEntry = unittest.mock.MagicMock(return_value = "user")
@@ -429,8 +399,8 @@ def test_hasUserValueUserChanges(global_stack):
     assert not global_stack.hasUserValue("infill_sparse_density")
     assert not global_stack.hasUserValue("")
 
-##  Tests whether the hasUserValue returns true for settings that are changed in
-#   the quality-changes container.
+
+##  Tests whether the hasUserValue returns true for settings that are changed in the quality-changes container.
 def test_hasUserValueQualityChanges(global_stack):
     container = unittest.mock.MagicMock()
     container.getMetaDataEntry = unittest.mock.MagicMock(return_value = "quality_changes")
@@ -441,8 +411,8 @@ def test_hasUserValueQualityChanges(global_stack):
     assert not global_stack.hasUserValue("infill_sparse_density")
     assert not global_stack.hasUserValue("")
 
-##  Tests whether a container in some other place on the stack is correctly not
-#   recognised as user value.
+
+##  Tests whether a container in some other place on the stack is correctly not recognised as user value.
 def test_hasNoUserValue(global_stack):
     container = unittest.mock.MagicMock()
     container.getMetaDataEntry = unittest.mock.MagicMock(return_value = "quality")
@@ -451,21 +421,25 @@ def test_hasNoUserValue(global_stack):
 
     assert not global_stack.hasUserValue("layer_height") #However this container is quality, so it's not a user value.
 
+
 ##  Tests whether inserting a container is properly forbidden.
 def test_insertContainer(global_stack):
     with pytest.raises(InvalidOperationError):
         global_stack.insertContainer(0, unittest.mock.MagicMock())
 
+
 ##  Tests whether removing a container is properly forbidden.
 def test_removeContainer(global_stack):
     with pytest.raises(InvalidOperationError):
         global_stack.removeContainer(unittest.mock.MagicMock())
 
+
 ##  Tests whether changing the next stack is properly forbidden.
 def test_setNextStack(global_stack):
     with pytest.raises(InvalidOperationError):
         global_stack.setNextStack(unittest.mock.MagicMock())
 
+
 ##  Tests setting properties directly on the global stack.
 @pytest.mark.parametrize("key,              property,         value", [
                         ("layer_height",    "value",          0.1337),
@@ -479,29 +453,7 @@ def test_setPropertyUser(key, property, value, global_stack):
     user_changes.getMetaDataEntry = unittest.mock.MagicMock(return_value = "user")
     global_stack.userChanges = user_changes
 
-    global_stack.setProperty(key, property, value) #The actual test.
+    global_stack.setProperty(key, property, value)  # The actual test.
 
-    global_stack.userChanges.setProperty.assert_called_once_with(key, property, value) #Make sure that the user container gets a setProperty call.
-
-##  Tests setting properties on specific containers on the global stack.
-@pytest.mark.parametrize("target_container,    stack_variable", [
-                        ("user",               "userChanges"),
-                        ("quality_changes",    "qualityChanges"),
-                        ("quality",            "quality"),
-                        ("material",           "material"),
-                        ("variant",            "variant"),
-                        ("definition_changes", "definitionChanges")
-])
-def test_setPropertyOtherContainers(target_container, stack_variable, global_stack):
-    #Other parameters that don't need to be varied.
-    key = "layer_height"
-    property = "value"
-    value = 0.1337
-    #A mock container in the right spot.
-    container = unittest.mock.MagicMock()
-    container.getMetaDataEntry = unittest.mock.MagicMock(return_value = target_container)
-    setattr(global_stack, stack_variable, container) #For instance, set global_stack.qualityChanges = container.
-
-    global_stack.setProperty(key, property, value, target_container = target_container) #The actual test.
-
-    getattr(global_stack, stack_variable).setProperty.assert_called_once_with(key, property, value) #Make sure that the proper container gets a setProperty call.
+    # Make sure that the user container gets a setProperty call.
+    global_stack.userChanges.setProperty.assert_called_once_with(key, property, value, None, False)
\ No newline at end of file
